<?php
if ( ! defined( 'ABSPATH' ) ) {
  exit; // Exit if accessed directly.
}

class Dig_Breadcrumb_Admin {
  public function init() {
    add_action( 'admin_menu', array( $this, 'add_admin_menu' ) );
    add_action( 'admin_init', array( $this, 'register_settings' ) );
  }

  public function add_admin_menu() {
    add_options_page(
      'dig Breadcrumb Settings',
      'dig Breadcrumb',
      'manage_options',
      'dig-breadcrumb',
      array( $this, 'create_admin_page' )
    );
  }

  public function register_settings() {
    register_setting( 'dig_breadcrumb_settings_group', 'dig_breadcrumb_settings' );

    add_settings_section(
      'dig_breadcrumb_main_section',
      esc_html__( 'Main Settings', 'dig-breadcrumb' ),
      null,
      'dig-breadcrumb'
    );

    add_settings_field(
      'home_label',
      esc_html__( 'Home Label', 'dig-breadcrumb' ),
      array( $this, 'home_label_callback' ),
      'dig-breadcrumb',
      'dig_breadcrumb_main_section'
    );

    add_settings_field(
      'search_label',
      esc_html__( 'Search Results Label', 'dig-breadcrumb' ),
      array( $this, 'search_label_callback' ),
      'dig-breadcrumb',
      'dig_breadcrumb_main_section'
    );

    add_settings_field(
      '404_label',
      esc_html__( '404 Label', 'dig-breadcrumb' ),
      array( $this, 'label_404_callback' ),
      'dig-breadcrumb',
      'dig_breadcrumb_main_section'
    );

    add_settings_field(
      'structured_data_format',
      esc_html__( 'Structured Data Format', 'dig-breadcrumb' ),
      array( $this, 'structured_data_format_callback' ),
      'dig-breadcrumb',
      'dig_breadcrumb_main_section'
    );

    add_settings_field(
      'top_level_label',
      esc_html__( 'Add Top Level Hierarchy (Label)', 'dig-breadcrumb' ),
      array( $this, 'top_level_label_callback' ),
      'dig-breadcrumb',
      'dig_breadcrumb_main_section'
    );

    add_settings_field(
      'top_level_url',
      esc_html__( 'Add Top Level Hierarchy (URL)', 'dig-breadcrumb' ),
      array( $this, 'top_level_url_callback' ),
      'dig-breadcrumb',
      'dig_breadcrumb_main_section'
    );

    add_settings_field(
      'separator',
      esc_html__( 'Separator', 'dig-breadcrumb' ),
      array( $this, 'separator_callback' ),
      'dig-breadcrumb',
      'dig_breadcrumb_main_section'
    );

    add_settings_field(
      'exclude_taxonomy',
      esc_html__( 'Exclude Taxonomy & Term', 'dig-breadcrumb' ),
      array( $this, 'exclude_taxonomy_callback' ),
      'dig-breadcrumb',
      'dig_breadcrumb_main_section'
    );

    add_settings_field(
      'custom_css',
      esc_html__( 'Custom CSS', 'dig-breadcrumb' ),
      array( $this, 'custom_css_callback' ),
      'dig-breadcrumb',
      'dig_breadcrumb_main_section'
    );
  }

  public function create_admin_page() {
    ?>
    <div class="wrap">
      <h1><?php esc_html_e( 'dig Breadcrumb Settings', 'dig-breadcrumb' ); ?></h1>
      <form method="post" action="options.php">
        <?php
          settings_fields( 'dig_breadcrumb_settings_group' );
          do_settings_sections( 'dig-breadcrumb' );
          submit_button();
        ?>
      </form>
    </div>
    <?php
  }

  public function home_label_callback() {
    $options = get_option( 'dig_breadcrumb_settings' );
    $home_label = isset( $options['home_label'] ) ? $options['home_label'] : esc_html__( 'Home', 'dig-breadcrumb' );
    ?>
    <input type="text" name="dig_breadcrumb_settings[home_label]" value="<?php echo esc_attr( $home_label ); ?>" placeholder="<?php esc_html_e( 'Home', 'dig-breadcrumb' ); ?>" class="regular-text" />
    <?php
  }

  public function search_label_callback() {
    $options = get_option( 'dig_breadcrumb_settings' );
    $search_label = isset( $options['search_label'] ) ? $options['search_label'] : esc_html__( 'Search results for:', 'dig-breadcrumb' );
    ?>
    <input type="text" name="dig_breadcrumb_settings[search_label]" value="<?php echo esc_attr( $search_label ); ?>" placeholder="<?php esc_html_e( 'Search results for:', 'dig-breadcrumb' ); ?>" class="regular-text" />
    <?php
  }

  public function label_404_callback() {
    $options = get_option( 'dig_breadcrumb_settings' );
    $label_404 = isset( $options['404_label'] ) ? $options['404_label'] : esc_html__( '404 Not Found', 'dig-breadcrumb' );
    ?>
    <input type="text" name="dig_breadcrumb_settings[404_label]" value="<?php echo esc_attr( $label_404 ); ?>" placeholder="<?php esc_html_e( '404 Not Found', 'dig-breadcrumb' ); ?>" class="regular-text" />
    <?php
  }

  public function structured_data_format_callback() {
    $options = get_option( 'dig_breadcrumb_settings' );
    $structured_data_format = isset( $options['structured_data_format'] ) ? $options['structured_data_format'] : 'microdata';
    ?>
    <select name="dig_breadcrumb_settings[structured_data_format]" class="regular-text">
      <option value="microdata" <?php selected( $structured_data_format, 'microdata' ); ?>><?php esc_html_e( 'Microdata', 'dig-breadcrumb' ); ?></option>
      <option value="json-ld" <?php selected( $structured_data_format, 'json-ld' ); ?>><?php esc_html_e( 'JSON-LD', 'dig-breadcrumb' ); ?></option>
    </select>
    <?php
  }

  public function top_level_label_callback() {
    $options = get_option( 'dig_breadcrumb_settings' );
    $top_level_label = isset( $options['top_level_label'] ) ? $options['top_level_label'] : '';
    ?>
    <input type="text" name="dig_breadcrumb_settings[top_level_label]" value="<?php echo esc_attr( $top_level_label ); ?>" placeholder="<?php esc_html_e( 'Top Level Label', 'dig-breadcrumb' ); ?>" class="regular-text" />
    <?php
  }

  public function top_level_url_callback() {
    $options = get_option( 'dig_breadcrumb_settings' );
    $top_level_url = isset( $options['top_level_url'] ) ? $options['top_level_url'] : '';
    ?>
    <input type="text" name="dig_breadcrumb_settings[top_level_url]" value="<?php echo esc_attr( $top_level_url ); ?>" placeholder="<?php esc_html_e( 'Top Level URL', 'dig-breadcrumb' ); ?>" class="regular-text" />
    <?php
  }

  public function separator_callback() {
    $options = get_option( 'dig_breadcrumb_settings' );
    $separator = isset( $options['separator'] ) ? $options['separator'] : '';
    ?>
    <input type="text" name="dig_breadcrumb_settings[separator]" value="<?php echo esc_attr( $separator ); ?>" placeholder="<?php esc_html_e( 'Separator', 'dig-breadcrumb' ); ?>" class="regular-text" />
    <?php
  }

  public function exclude_taxonomy_callback() {
    $options = get_option( 'dig_breadcrumb_settings' );
    $exclude_taxonomy = isset( $options['exclude_taxonomy'] ) ? $options['exclude_taxonomy'] : array();
    $post_types = get_post_types( array( 'public' => true ), 'objects' );

    foreach ( $post_types as $post_type ) {
      $taxonomies = get_object_taxonomies( $post_type->name, 'objects' );
      if ( !empty( $taxonomies ) ) { // ^N\m~[݂ꍇ̂ݓe^Cv\
        echo '<h3>' . esc_html( $post_type->label ) . '</h3>';
        foreach ( $taxonomies as $taxonomy ) {
          $checked = in_array( $taxonomy->name, $exclude_taxonomy ) ? 'checked' : '';
          ?>
          <label>
            <input type="checkbox" name="dig_breadcrumb_settings[exclude_taxonomy][]" value="<?php echo esc_attr( $taxonomy->name ); ?>" <?php echo esc_attr( $checked ); ?> />
            <?php echo esc_html( $taxonomy->label ); ?>
          </label><br />
          <?php
          $terms = get_terms( array(
            'taxonomy' => $taxonomy->name,
            'hide_empty' => false,
            'parent' => 0,
          ) );
          foreach ( $terms as $term ) {
            $this->print_term_checkboxes( $term, $exclude_taxonomy, 1 );
          }
        }
      }
    }
  }

  private function print_term_checkboxes( $term, $exclude_taxonomy, $level ) {
    $checked = in_array( $term->term_id, $exclude_taxonomy ) ? 'checked' : '';
    ?>
    <label style="margin-left: <?php echo esc_attr( $level * 20 ); ?>px;">
      <input type="checkbox" name="dig_breadcrumb_settings[exclude_taxonomy][]" value="<?php echo esc_attr( $term->term_id ); ?>" <?php echo esc_attr( $checked ); ?> />
      <?php echo esc_html( $term->name ); ?>
    </label><br />
    <?php
    $child_terms = get_terms( array(
      'taxonomy' => $term->taxonomy,
      'hide_empty' => false,
      'parent' => $term->term_id,
    ) );
    foreach ( $child_terms as $child_term ) {
      $this->print_term_checkboxes( $child_term, $exclude_taxonomy, $level + 1 );
    }
  }

  public function custom_css_callback() {
    $options = get_option( 'dig_breadcrumb_settings' );
    $custom_css = isset( $options['custom_css'] ) ? $options['custom_css'] : '';
    ?>
    <textarea name="dig_breadcrumb_settings[custom_css]" class="large-text" rows="10" placeholder="<?php esc_attr( 'Custom CSS', 'dig-breadcrumb' ); ?>"><?php echo esc_textarea( $custom_css ); ?></textarea>
    <p><?php esc_html_e( 'Example HTML structure:', 'dig-breadcrumb' ); ?></p>
    <pre>
&lt;nav aria-label="breadcrumb" itemscope itemtype="http://schema.org/BreadcrumbList"&gt;
  &lt;ol class="dig-breadcrumb"&gt;
    &lt;li class="breadcrumb-item" itemprop="itemListElement" itemscope itemtype="http://schema.org/ListItem"&gt;
      &lt;a href="#" itemprop="item"&gt;&lt;span itemprop="name"&gt;Home&lt;/span&gt;&lt;/a&gt;
      &lt;meta itemprop="position" content="1" /&gt;
    &lt;/li&gt;
    &lt;li class="breadcrumb-item" itemprop="itemListElement" itemscope itemtype="http://schema.org/ListItem"&gt;
      &lt;a href="#" itemprop="item"&gt;&lt;span itemprop="name"&gt;Category&lt;/span&gt;&lt;/a&gt;
      &lt;meta itemprop="position" content="2" /&gt;
    &lt;/li&gt;
    &lt;li class="breadcrumb-item active" itemprop="itemListElement" itemscope itemtype="http://schema.org/ListItem"&gt;
      &lt;span itemprop="name"&gt;Post Title&lt;/span&gt;
      &lt;meta itemprop="position" content="3" /&gt;
    &lt;/li&gt;
  &lt;/ol&gt;
&lt;/nav&gt;
    </pre>
    <?php
  }
}
