<?php
if ( ! defined( 'ABSPATH' ) ) {
  exit; // Exit if accessed directly.
}

class Dig_Breadcrumb {
  public function run() {
    add_shortcode( 'dig_breadcrumb', array( $this, 'display_breadcrumb' ) );
    add_action( 'wp_head', array( $this, 'custom_css' ) );
  }

  public function display_breadcrumb() {
    $options = get_option( 'dig_breadcrumb_settings' );

    $home_label = isset( $options['home_label'] ) && !empty( $options['home_label'] ) ? $options['home_label'] : __( 'Home', 'dig-breadcrumb' );
    $search_label = isset( $options['search_label'] ) && !empty( $options['search_label'] ) ? $options['search_label'] : __( 'Search results for:', 'dig-breadcrumb' );
    $not_found_label = isset( $options['404_label'] ) && !empty( $options['404_label'] ) ? $options['404_label'] : __( '404 Not Found', 'dig-breadcrumb' );
    $structured_data_format = isset( $options['structured_data_format'] ) ? $options['structured_data_format'] : 'microdata';
    $top_level_label = isset( $options['top_level_label'] ) && !empty( $options['top_level_label'] ) ? $options['top_level_label'] : '';
    $top_level_url = isset( $options['top_level_url'] ) && !empty( $options['top_level_url'] ) ? $options['top_level_url'] : '';
    $separator = isset( $options['separator'] ) ? $options['separator'] : '';
    $exclude_taxonomy = isset( $options['exclude_taxonomy'] ) ? $options['exclude_taxonomy'] : array();

    global $post;
    $breadcrumb = '';

    if ( $structured_data_format === 'microdata' ) {
      $breadcrumb .= '<nav aria-label="breadcrumb" itemscope itemtype="http://schema.org/BreadcrumbList"><ol class="dig-breadcrumb">';
    } else {
      $breadcrumb .= '<nav aria-label="breadcrumb"><ol class="dig-breadcrumb">';
    }

    $position = 1;

    // トップレベルの項目を追加
    if ( ! empty( $top_level_label ) && ! empty( $top_level_url ) ) {
      $breadcrumb .= $this->get_breadcrumb_item( $top_level_url, $top_level_label, $position, $structured_data_format );
      $position++;
      if ( $separator ) {
        $breadcrumb .= '<li class="breadcrumb-separator">' . esc_html( $separator ) . '</li>';
      }
    }

    $breadcrumb .= $this->get_breadcrumb_item( home_url(), $home_label, $position, $structured_data_format );
    $position++;
    if ( $separator ) {
      $breadcrumb .= '<li class="breadcrumb-separator">' . esc_html( $separator ) . '</li>';
    }

    if ( is_singular() ) {
      $post_type = get_post_type_object( get_post_type() );
      if ( $post_type->has_archive ) {
        $breadcrumb .= $this->get_breadcrumb_item( get_post_type_archive_link( $post_type->name ), $post_type->label, $position, $structured_data_format );
        $position++;
        if ( $separator ) {
          $breadcrumb .= '<li class="breadcrumb-separator">' . esc_html( $separator ) . '</li>';
        }
      }

      // タクソノミーのタームを取得して表示
      $taxonomies = get_object_taxonomies( get_post_type(), 'objects' );
      foreach ( $taxonomies as $taxonomy ) {
        if ( ! in_array( $taxonomy->name, $exclude_taxonomy ) ) {
          $terms = wp_get_post_terms( get_the_ID(), $taxonomy->name );
          foreach ( $terms as $term ) {
            if ( ! in_array( $term->term_id, $exclude_taxonomy ) ) {
              $ancestors = get_ancestors( $term->term_id, $taxonomy->name );
              $ancestors = array_reverse( $ancestors );
              foreach ( $ancestors as $ancestor_id ) {
                if ( ! in_array( $ancestor_id, $exclude_taxonomy ) ) {
                  $ancestor = get_term( $ancestor_id, $taxonomy->name );
                  $breadcrumb .= $this->get_breadcrumb_item( get_term_link( $ancestor ), $ancestor->name, $position, $structured_data_format );
                  $position++;
                  if ( $separator ) {
                    $breadcrumb .= '<li class="breadcrumb-separator">' . esc_html( $separator ) . '</li>';
                  }
                }
              }
              $breadcrumb .= $this->get_breadcrumb_item( get_term_link( $term ), $term->name, $position, $structured_data_format );
              $position++;
              if ( $separator ) {
                $breadcrumb .= '<li class="breadcrumb-separator">' . esc_html( $separator ) . '</li>';
              }
            }
          }
        }
      }

      $breadcrumb .= $this->get_breadcrumb_item( get_permalink(), get_the_title(), $position, $structured_data_format, true );
    } elseif ( is_category() || is_tag() || is_tax() ) {
      $term = get_queried_object();
      if ( ! in_array( $term->term_id, $exclude_taxonomy ) ) {
        if ( $term->parent != 0 ) {
          $parent = get_term( $term->parent );
          if ( ! in_array( $parent->term_id, $exclude_taxonomy ) ) {
            $breadcrumb .= $this->get_breadcrumb_item( get_term_link( $parent ), $parent->name, $position, $structured_data_format );
            $position++;
            if ( $separator ) {
              $breadcrumb .= '<li class="breadcrumb-separator">' . esc_html( $separator ) . '</li>';
            }
          }
        }
        $breadcrumb .= $this->get_breadcrumb_item( get_term_link( $term ), $term->name, $position, $structured_data_format, true );
      }
    } elseif ( is_archive() ) {
      if ( is_post_type_archive() ) {
        $post_type = get_post_type_object( get_post_type() );
        $breadcrumb .= $this->get_breadcrumb_item( get_post_type_archive_link( $post_type->name ), $post_type->label, $position, $structured_data_format, true );
      } elseif ( is_date() ) {
        if ( is_day() ) {
          $breadcrumb .= $this->get_breadcrumb_item( get_day_link( get_the_date('Y'), get_the_date('m'), get_the_date('d') ), get_the_date(), $position, $structured_data_format, true );
        } elseif ( is_month() ) {
          $breadcrumb .= $this->get_breadcrumb_item( get_month_link( get_the_date('Y'), get_the_date('m') ), get_the_date( 'F Y' ), $position, $structured_data_format, true );
        } elseif ( is_year() ) {
          $breadcrumb .= $this->get_breadcrumb_item( get_year_link( get_the_date('Y') ), get_the_date( 'Y' ), $position, $structured_data_format, true );
        }
      }
    } elseif ( is_search() ) {
      $breadcrumb .= $this->get_breadcrumb_item( get_search_link(), $search_label . ' ' . get_search_query(), $position, $structured_data_format, true );
    } elseif ( is_404() ) {
      $breadcrumb .= $this->get_breadcrumb_item( '', $not_found_label, $position, $structured_data_format, true );
    }

    $breadcrumb .= '</ol></nav>';

    if ( $structured_data_format === 'json-ld' && ! is_404() ) {
      $breadcrumb .= $this->generate_json_ld();
    }

    return $breadcrumb;
  }

  private function get_breadcrumb_item( $url, $name, $position, $structured_data_format, $is_last = false ) {
    if ( $structured_data_format === 'json-ld' ) {
      return '<li class="breadcrumb-item">' . ( $is_last ? '<span>' . esc_html( $name ) . '</span>' : '<a href="' . esc_url( $url ) . '">' . esc_html( $name ) . '</a>' ) . '</li>';
    } else {
      return '<li class="breadcrumb-item" itemprop="itemListElement" itemscope itemtype="http://schema.org/ListItem">' . 
        ( $is_last ? '<span itemprop="name">' . esc_html( $name ) . '</span>' : '<a href="' . esc_url( $url ) . '" itemprop="item"><span itemprop="name">' . esc_html( $name ) . '</span></a>' ) .
        '<meta itemprop="position" content="' . esc_attr( $position ) . '" />' .
        '</li>';
    }
  }

  private function generate_json_ld() {
    $options = get_option( 'dig_breadcrumb_settings' );

    $home_label = isset( $options['home_label'] ) && !empty( $options['home_label'] ) ? $options['home_label'] : __( 'Home', 'dig-breadcrumb' );
    $top_level_label = isset( $options['top_level_label'] ) && !empty( $options['top_level_label'] ) ? $options['top_level_label'] : '';
    $top_level_url = isset( $options['top_level_url'] ) && !empty( $options['top_level_url'] ) ? $options['top_level_url'] : '';
    $exclude_taxonomy = isset( $options['exclude_taxonomy'] ) ? $options['exclude_taxonomy'] : array();
    $position = 1;

    $json_ld = [
      "@context" => "https://schema.org",
      "@type" => "BreadcrumbList",
      "itemListElement" => []
    ];

    if ( ! empty( $top_level_label ) && ! empty( $top_level_url ) ) {
      $json_ld['itemListElement'][] = [
        "@type" => "ListItem",
        "position" => $position,
        "name" => esc_html( $top_level_label ),
        "item" => esc_url( $top_level_url )
      ];
      $position++;
    }

    $json_ld['itemListElement'][] = [
      "@type" => "ListItem",
      "position" => $position,
      "name" => esc_html( $home_label ),
      "item" => esc_url( home_url() )
    ];
    $position++;

    if ( is_singular() ) {
      global $post;
      $post_type = get_post_type_object( get_post_type() );
      if ( $post_type->has_archive ) {
        $json_ld['itemListElement'][] = [
          "@type" => "ListItem",
          "position" => $position,
          "name" => esc_html( $post_type->label ),
          "item" => esc_url( get_post_type_archive_link( $post_type->name ) )
        ];
        $position++;
      }

      $taxonomies = get_object_taxonomies( get_post_type(), 'objects' );
      foreach ( $taxonomies as $taxonomy ) {
        if ( ! in_array( $taxonomy->name, $exclude_taxonomy ) ) {
          $terms = wp_get_post_terms( get_the_ID(), $taxonomy->name );
          foreach ( $terms as $term ) {
            if ( ! in_array( $term->term_id, $exclude_taxonomy ) ) {
              $ancestors = get_ancestors( $term->term_id, $taxonomy->name );
              $ancestors = array_reverse( $ancestors );
              foreach ( $ancestors as $ancestor_id ) {
                if ( ! in_array( $ancestor_id, $exclude_taxonomy ) ) {
                  $ancestor = get_term( $ancestor_id, $taxonomy->name );
                  $json_ld['itemListElement'][] = [
                    "@type" => "ListItem",
                    "position" => $position,
                    "name" => esc_html( $ancestor->name ),
                    "item" => esc_url( get_term_link( $ancestor ) )
                  ];
                  $position++;
                }
              }
              $json_ld['itemListElement'][] = [
                "@type" => "ListItem",
                "position" => $position,
                "name" => esc_html( $term->name ),
                "item" => esc_url( get_term_link( $term ) )
              ];
              $position++;
            }
          }
        }
      }

      $json_ld['itemListElement'][] = [
        "@type" => "ListItem",
        "position" => $position,
        "name" => esc_html( get_the_title() ),
        "item" => esc_url( get_permalink() )
      ];
    } elseif ( is_category() || is_tag() || is_tax() ) {
      $term = get_queried_object();
      if ( ! in_array( $term->term_id, $exclude_taxonomy ) ) {
        if ( $term->parent != 0 ) {
          $parent = get_term( $term->parent );
          if ( ! in_array( $parent->term_id, $exclude_taxonomy ) ) {
            $json_ld['itemListElement'][] = [
              "@type" => "ListItem",
              "position" => $position,
              "name" => esc_html( $parent->name ),
              "item" => esc_url( get_term_link( $parent ) )
            ];
            $position++;
          }
        }
        $json_ld['itemListElement'][] = [
          "@type" => "ListItem",
          "position" => $position,
          "name" => esc_html( $term->name ),
          "item" => esc_url( get_term_link( $term ) )
        ];
      }
    } elseif ( is_archive() ) {
      if ( is_post_type_archive() ) {
        $post_type = get_post_type_object( get_post_type() );
        $json_ld['itemListElement'][] = [
          "@type" => "ListItem",
          "position" => $position,
          "name" => esc_html( $post_type->label ),
          "item" => esc_url( get_post_type_archive_link( $post_type->name ) )
        ];
      } elseif ( is_date() ) {
        if ( is_day() ) {
          $json_ld['itemListElement'][] = [
            "@type" => "ListItem",
            "position" => $position,
            "name" => esc_html( get_the_date() ),
            "item" => esc_url( get_day_link( get_the_date('Y'), get_the_date('m'), get_the_date('d') ) )
          ];
        } elseif ( is_month() ) {
          $json_ld['itemListElement'][] = [
            "@type" => "ListItem",
            "position" => $position,
            "name" => esc_html( get_the_date( 'F Y' ) ),
            "item" => esc_url( get_month_link( get_the_date('Y'), get_the_date('m') ) )
          ];
        } elseif ( is_year() ) {
          $json_ld['itemListElement'][] = [
            "@type" => "ListItem",
            "position" => $position,
            "name" => esc_html( get_the_date( 'Y' ) ),
            "item" => esc_url( get_year_link( get_the_date('Y') ) )
          ];
        }
      }
    } elseif ( is_search() ) {
      $json_ld['itemListElement'][] = [
        "@type" => "ListItem",
        "position" => $position,
        "name" => esc_html( $search_label ) . ' ' . esc_html( get_search_query() ),
        "item" => esc_url( get_search_link() )
      ];
    }

    return '<script type="application/ld+json">' . wp_json_encode( $json_ld, JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES | JSON_PRETTY_PRINT ) . '</script>';
  }

  public function custom_css() {
    $options = get_option( 'dig_breadcrumb_settings' );
    $custom_css = isset( $options['custom_css'] ) ? $options['custom_css'] : '';
    if ( ! empty( $custom_css ) ) {
      echo '<style type="text/css">' . esc_html( $custom_css ) . '</style>';
    }
  }
}
